/******************************************************************************/
/*                                                                            */
/* IBDgraph.h: Header file for the IBDGraph library.                          */
/*                                                                            */
/* This header file contains #includes for header files for libraries used    */
/* and #defines, typedefs, variables and subroutine prototypes.               */
/*                                                                            */
/******************************************************************************/

#ifndef IBDGRAPH_H
#define IBDGRAPH_H

#define EDGE_NAME_BUFFER_SIZE 64
#define _UTILITIES_H_

/* Include all the necessary header files from the IBDgraph directory.        */

#include "./../IBDgraph/ibdstructures.h"
#include "./../IBDgraph/hashkeys.h"
#include "./../IBDgraph/hashobject.h"
#include "./../IBDgraph/hashtable.h"
#include "./../IBDgraph/markerinfo.h"
#include "./../IBDgraph/object.h"

struct lodscore_element {               /* Lodscore_element:                  */
    int need_loglike;                   /* 1 -> Need to calculate loglike.    */
    /*                                     0 -> No longer need to calculate   */
    /*                                          this element's loglike again. */
    int num_refs;                       /* Number of pointers to this         */
    /*                                     lodscore_element.                  */
    double *loglike;                     /* Pointer to currentlog likelihoods */
                        };
typedef struct lodscore_element LSCORE_ELEM;
typedef LSCORE_ELEM * LSE_PTR;

struct marker_classes {                 /* marker_classes:                    */
    long mark_num;                      /* Original marker number.            */
    long num_classes;                   /* Number of classes.                 */
    long *num_graphs;                   /* Number of graphs in each class.    */
    LSE_PTR *p_lse;                     /* Pointers to the lodscore_elements  */
    /*                                     for each class at this marker.     */
                      };
typedef struct marker_classes MARK_CLASS;

typedef MarkerInfo *MI_PTR;             /* Pointer to a MarkerInfo object.    */

struct mi_list_node {                   /* mi_list_node:                      */
    MI_PTR p_mi;                        /* Pointer to the MarkerInfo object.  */
    struct mi_list_node * p_next;       /* Pointer to next mi_list_node.      */
                    };
typedef struct mi_list_node MI_LIST;
typedef MI_LIST * MI_LIST_NODE_PTR;

#ifdef  DEFINE_DATA
GLOBAL MI_LIST_NODE_PTR first_mi_list_node = NULL;
#else
GLOBAL MI_LIST_NODE_PTR first_mi_list_node;
#endif  /*DEFINE_DATA*/

/* Function prototypes.                                                       */
void createIBDGraphs (char *, IBDGraphList *);

/* The following prototype belongs in ibdstructures.h.  Leave it here until   */
/* it can be added to ibdstructures.h.  SML:  November 17, 2011.              */
void  IBDGraph_Refresh (IBDGraph *);

void  PrintInterval (markertype, markertype);
MI_LIST_NODE_PTR alloc_mi_list_node (MI_PTR);
void  free_mi_list (void);

#endif /*IBDGRAPH_H*/
