/******************************************************************************/
/*                                                                            */
/* Created from create_recycle (which was from create).                       */
/* Output from beaglesim + simpop_fgl -> haplotypes.                          */
/*                                                                            */
/******************************************************************************/

#define PNAM "ibd_create"

#include "parsie.h"
#include "rans.h"
#include "ibd_create.h"

/******************************************************************************/
/*                                                                            */
/* The fgl2haplo subprogram main routine.                                     */
/*                                                                            */
/******************************************************************************/

int fgl2haplo (int argc, char **argv)
{
   FILE *sim_chroms = NULL;
   FILE *snps_file = NULL;
   FILE *oScorF = NULL;
   FILE *oSeedF;

   Hashnode **hashtable;

   char **assigned = NULL;
   char **snps = NULL;

   char *input_file;

   int   nsnps;                         /* Number of SNPs in the snps matrix. */
   int   nhaps;                         /* Number of haplotypes in the snps   */
   /*                                      matrix.                            */

   int   num_uniq_fgls;                 /* Count of number of unique FGL's.   */

   int   i1;                            /* Subscript to current individual.   */
   int   m1;                            /* Subscript to current marker.       */
   int   h1;                            /* First haplotype subscript.         */
   int   h2;                            /* Second haplotype subscript.        */

   int   rc = 0;                        /* Return code from populate_snps.    */

   /* ................................  SET-UP   ............................ */

   setupBasic ("AR", argc, argv, PNAM);

   /* Tell the user what scale factor for the conversion of centimorgans to   */
   /* million base pairs is being used in this run of fgl2haplo.              */

   printf (" The scale factor used for this run of fgl2haplo is");
   printf (" %7.4f Mbp per cM\n\n", MBPperCM);

   /* Get the file name of the input ibd_graph gamete data file.              */

   input_file = get_filent (IGAM)->fname;
   if (input_file == NULL)
      give_up (ERD_igd);

   /* Open the Morgan input gamete data file to use as fgl2haplo's input      */
   /* ibd_graph gamete data file, with the simulated FGL's included.          */

   if (sim_chroms == NULL)
      if (!(sim_chroms = open_fil (IGAM)))
         give_up (ERR_qit);

   if (!INP_GENOTYPES)
   {
      /* If we are not using Morgan's "set marker data ..." as genotypes, we  */
      /* need to open an input extra file for reading founder haplotypes.     */

      if (snps_file == NULL)
         if (!(snps_file = open_fil (XTRA)))
            give_up (ERR_qit);

      nsnps = NumbSnps;
   }
   else if (INP_GENOTYPES)
      nsnps = NMarks;

   /* Open the output scores file to use as fgl2haplo's results file.         */

   if (oScorF == NULL)
      if (!(oScorF = open_fil (OSCORE)))
         give_up (ERR_qit);

   fprintf (stdout, "\n");
   fflush (stdout);

   /* Call init_snp_pos() to intialize the SNP positions vector, snp_pos.     */

   init_snp_pos (nsnps);

   /* Populate the array of snps in the fashion determined by the HaploInput  */
   /* global variable.                                                        */

   if (INP_GENOTYPES)
   {
      /* Populate the snps matrix using the way the data are normally found   */
      /* in Morgan.  For this case the data have already been read in from a  */
      /* Morgan parameter file or marker data file.  They just need to be     */
      /* copied from the ObsMark array to the snps matrix.                    */

      fprintf (stdout, " Using marker data as haplotypes\n");
      fflush (stdout);

      /* Each observed individual has 2 haplotypes.  So we can compute the    */
      /* number of haplotypes directly from the Morgan data.                  */

      nhaps = NObs * 2;

      /* Since the populate_snps subroutine is not being called in this case, */
      /* it is necessary to allocate the storage for the snps matrix.         */

      /* Do the same as calling the alloc_char2d subroutine except add one    */
      /* extra character at the end of the array to leave room for the null   */
      /* character appended when the sprintf subroutine is called.            */

      {
         snps = (char **) malloc (nsnps * sizeof (char *));
         alloc_chk (WHERE, snps);
         snps[0] = (char *) malloc ((nsnps * nhaps * sizeof (char)) + 1);
         alloc_chk (WHERE, snps[0]);
         for (m1 = 1; m1 < nsnps; ++m1)
            snps[m1] = snps[m1 - 1] + nhaps;
         memset (snps[0], 0, ((nsnps * nhaps * sizeof (char)) + 1));
      }

      for (m1 = 0; m1 < nsnps; ++m1)
      {
         for (i1 = 0; i1 < NObs; ++i1)
         {
            h1 = i1 * 2;                /* Compute subscripts to haplotypes.  */
            h2 = h1 + 1;

            sprintf (&snps[m1][h1], "%1d", ObsMark[i1][m1][0]);
            sprintf (&snps[m1][h2], "%1d", ObsMark[i1][m1][1]);
         }
      }
   }
   else
   {
      /* Reading in the founder haplotype data.                               */

      fprintf (stdout, " Reading in the founder haplotypes\n");
      fflush (stdout);

      nhaps = NumbHaps;

      if (INP_HAPS_AS_ROWS)
      {
         /* The following line populates snps assuming each row of the input  */
         /* file contains all the SNPs for a single haplotype.                */

         rc = populate_snps ("SE", snps_file, nsnps, nhaps, &snps);
      }
      else if (INP_HAPS_AS_COLS)
      {
         /* The following line populates snps the way the data are normally   */
         /* output by beagle.  Each row of the input file contains all the    */
         /* haplotypes (ie alleles) for a single SNP.                         */

         rc = populate_snps ("HE", snps_file, nsnps, nhaps, &snps);
      }
   }

   if (rc != 0)
   {
      fprintf (stderr, "\n An error occurred while reading %s\n", input_file);
      fprintf (stderr, " The return code from populate_snps was %d.\n\n", rc);
      fflush (stderr);

      return rc;
   }

   if (snps_file)
      fclose (snps_file);

   /* Read in ALL the input ibd_graph gamete data from simpop_fgl's output.   */

   fprintf (stdout, "\n Reading in the FGL chromosomes\n");
   fflush (stdout);

   read_gamete_data (sim_chroms, &num_gametes, &ibdgraph_gametes);
   fclose (sim_chroms);

   /* Build the FGL hash table.                                               */

   hashtable = alloc_hashtable ();
   fill_hashtable (hashtable, nsnps, &num_uniq_fgls);

   /* Make sure that there are enough founder haplotypes in the snps array to */
   /* accomodate all the FGL's read in from the input ibd_graph gamete data   */
   /* file.                                                                   */

   if (num_uniq_fgls > nhaps)
   {
      if (DebugLevel == 4)
         print_table (stderr, hashtable);

      fprintf (stderr, "\n\n");
      fprintf (stderr, " Error: There are not enough founder haplotypes");
      fprintf (stderr, " to cover all the FGLs (E)");
      fprintf (stderr, "\n\n");
      fflush (stderr);

      give_up (ERR_qit);
   }

   /* Call the assign_alleles subroutine to create the assigned matrix, which */
   /* will contain the alleles assigned for all haplotypes.                   */

   assigned = assign_alleles (hashtable, snps, nsnps, nhaps);

   /* Check on how the user wants the assigned haplotypes output.             */

   if (OUT_GENOTYPES)
   {
      /* Call the output_genotypes subroutine to write the output as          */
      /* genotypes so they can be input to Morgan programs.                   */

      output_genotypes (oScorF, assigned, nsnps, nhaps);
   }
   else
   {
      /* Otherwise, call the output_haplotypes subroutine to write the output */
      /* haplotypes to the oScorF file.                                       */

      output_haplotypes (oScorF, assigned, nsnps, nhaps);
   }

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   fflush (stdout);
   fflush (oScorF);
   fclose (oScorF);

   /* We are done, so free up the storage that has been allocated by the      */
   /* fgl2haplo subprogram.                                                   */

   free_gamete_data (num_gametes, ibdgraph_gametes);
   free_hashtable (hashtable);
   free_marek ();
   free_auto ();
   free_locnams ();
   free_fnams ();

   /* Free the remaining fgl2haplo subroutine allocated memory.               */

   free_2d ((void *) &assigned);
   free_2d ((void *) &snps);
   free_1d ((void *) &snp_pos);

   return 0;
}
