/* checks if x has only one bit set (only one state allowed) */
#define ONLY_ONE_BIT(x) ((x) && !((x) & ((x) - 1)))
/* checks if x has bit n set */
#define IS_SET(x,n) ((x)&(1<<(n)))

/* array of bitmasks indicating allowed states between 
 * AC given states [AB][BC].
 * example: AB are in state 2 (1-index), BC are in state 5,
 * AC can be in 3, 4, or 5, so the mask is
 * 000000000011100 = 28 = trioMasks[2][5]
*/
unsigned int trioMasks[15][15] =
   { {1, 2, 4, 8, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
{0, 0, 0, 0, 0, 1, 2, 2, 4, 8, 16, 4, 16, 8, 16},
{0, 0, 0, 0, 0, 8, 4, 16, 2, 1, 2, 16, 4, 8, 16},
{8, 20, 18, 9, 22, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
{0, 0, 0, 0, 0, 8, 16, 20, 16, 8, 20, 18, 18, 9, 22},
{32, 192, 2304, 8704, 21504, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
{0, 0, 0, 0, 0, 32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384},
{0, 0, 0, 0, 0, 32, 128, 192, 2048, 8192, 20480, 2304, 17408, 8704, 21504},
{0, 0, 0, 0, 0, 512, 256, 1024, 64, 32, 128, 4096, 2048, 8192, 16384},
{512, 1280, 4160, 8224, 18560, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
{0, 0, 0, 0, 0, 512, 1024, 1280, 4096, 8192, 18432, 4160, 16512, 8224, 18560},
{0, 0, 0, 0, 0, 8192, 2048, 20480, 128, 32, 192, 17408, 2304, 8704, 21504},
{0, 0, 0, 0, 0, 8192, 4096, 18432, 1024, 512, 1280, 16512, 4160, 8224, 18560},
{8192, 22528, 17536, 8736, 24000, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
{0, 0, 0, 0, 0, 8192, 16384, 22528, 16384, 8192, 22528, 17536, 17536, 8736,
 24000}
};

/* masks for AC given state of AB; only restricts
 * so that autozygosity of A is preserved */
unsigned int autozygMasks[15] =
   { 31, 31, 31, 31, 31, 32736, 32736, 32736, 32736, 32736, 32736, 32736, 32736,
   32736, 32736
};

/* if AB are in state n, BA are in state flipStates[n] */
int   flippedStates[15] = { 0, 5, 9, 3, 13, 1, 6, 7, 8, 2, 11, 10, 12, 4, 14 };
int   unflippedStates[15] =
   { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14 };

int  *arrayAB;
int  *arrayBC;

void hap_fwd_compute_cond (int *hd, int nd, int mph, int ms, double *allefrq,
                           double *fff, double *mdist, double **pprob, int igdx,
                           int *phasvec, unsigned int *maskvec, int num_haps)
/* same as hap_fwd_compute, but with a vector masking disallowed states
 */
{
   double qn;
   double *pen2;

   int   i, ii, j, jj;
   int   phasint = 0;

   pen2 = alloc_dbl1d (ms);

   printf ("\n Starting fwd-compute for %d markers", nd);
   fflush (stdout);

   /* If we are using the phase info, phasvec ought to be a non-null pointer. */
   if (igdx == 2)
      phasint = phasvec[0];

   hap_set_penmat (allefrq[0], pen2, num_haps, ms, hd[0], GenoErrRat,
                   igdx, phasint, state_matrix);

   if (maskvec[0] == 0)
   {
      fprintf (stderr, "No legal state at locus %d!\n", 0);
      exit (EXIT_FAILURE);              /* this should never happen */
   }
   qn = 0;
   for (j = 0; j < ms; ++j)
   {
      if (IS_SET (maskvec[0], j))
      {
         pprob[0][j] = fff[j] * pen2[j];
         qn += pprob[0][j];
      }
   }
   for (j = 0; j < ms; ++j)
   {
      pprob[0][j] /= qn;
   }

   for (i = 1; i < nd; ++i)
   {
      qn = 0.0;

      ii = skipmark * i;
      if (igdx == 2)
         phasint = phasvec[ii];
      hap_set_penmat (allefrq[ii], pen2, num_haps, ms, hd[i], GenoErrRat,
                      igdx, phasint, state_matrix);
      set_transmat (ms, transmat, fff, mdist[i - 1]);

      for (j = 0; j < ms; ++j)
      {
         if (IS_SET (maskvec[i], j))
         {
            pprob[i][j] = 0;
            for (jj = 0; jj < ms; ++jj)
            {
               if (IS_SET (maskvec[i - 1], jj))
               {
                  pprob[i][j] += pprob[i - 1][jj] * transmat[jj][j] * pen2[j];
               }
            }
            qn += pprob[i][j];
         }
      }
      for (j = 0; j < ms; ++j)
      {
         pprob[i][j] = pprob[i][j] / qn;
      }
   }

   free_1d ((void *) &pen2);
   return;
}

void hap_sample_cond (int nd, int ms, int *simStates,
                      double *fff, double *mdist, double **pprob,
                      unsigned int *maskvec)
/* backwards sampling of the entire sequence of IBD states
 * given marker data: the final forward probabilities are
 * Pr(state = S | Y = full data), so first sample this S.
 * Then given S_[j..n], Pr(S_[j-1] | S_[j..n],Y) =
 * Pr(S_[j-1] | S_j, Y_[1..j-1]) (by conditional independence)
 * proportional to Pr(S_j | S_[j-1])*Pr(S_[j-1]|Y_[1..j-1])
 * (by Bayes thm). The second factor is the forward prob for s
 * at locus j-1. */
{
   int   i, ii, j;
   double *qi, qn, sampleThreshold, sampleSum;
   double *pen2;

   pen2 = alloc_dbl1d (ms);
   qi = alloc_dbl1d (ms);

   printf ("\n Starting back-simulation for %d markers", nd);
   fflush (stdout);

   if (ONLY_ONE_BIT (maskvec[nd - 1]))
   {
      for (j = 0; j < ms; j++)
      {
         if (IS_SET (maskvec[nd - 1], j))
         {
            simStates[nd - 1] = j;
            break;
         }
      }
   }
   else
   {
      sampleSum = 0;
      sampleThreshold = rans ();
      for (j = 0; j < ms; ++j)
      {
         if (IS_SET (maskvec[nd - 1], j))
         {
            sampleSum += pprob[nd - 1][j];
            if (sampleSum > sampleThreshold)
            {
               simStates[nd - 1] = j;
               break;
            }
         }
      }
   }

   ii = (nd - 1) * skipmark;

   for (i = nd - 2; i >= 0; --i)
   {
      /* if only one state allowed, that's the state */
      if (ONLY_ONE_BIT (maskvec[i]))
      {
         for (j = 0; j < ms; j++)
         {
            if (IS_SET (maskvec[i], j))
            {
               simStates[i] = j;
               break;
            }
         }
         continue;
      }
      set_transmat (ms, transmat, fff, mdist[i]);
      for (j = 0; j < ms; ++j)
      {
         if (IS_SET (maskvec[i], j))
         {
            qi[j] = transmat[j][simStates[i + 1]] * pprob[i][j];
         }
      }
      qn = 0.0;
      for (j = 0; j < ms; ++j)
      {
         if (IS_SET (maskvec[i], j))
         {
            qn += qi[j];
         }
      }
      for (j = 0; j < ms; ++j)
      {
         if (IS_SET (maskvec[i], j))
         {
            qi[j] = qi[j] / qn;
         }
      }

      sampleSum = 0;
      sampleThreshold = rans ();
      for (j = 0; j < ms; ++j)
      {
         if (IS_SET (maskvec[i], j))
         {
            sampleSum += qi[j];
            if (sampleSum > sampleThreshold)
            {
               simStates[i] = j;
               break;
            }
         }
      }

   }

   free_1d ((void *) &pen2);
   free_1d ((void *) &qi);
}

void makeDataIndexed (int *hd, int **hm, int k1, int k2, int nd)
/* Make data length nd from  indivs k1 and k2 .
 * Only haplotypic data.
 */
{
   int   i, j;
   for (i = 0; i < nd; ++i)
   {
      hd[i] = 0;
      for (j = 0; j < 2; ++j)
      {
         if (hm[2 * k1 + j][i] == 0)
         {
            hd[i] = -1;
            break;
         }
         hd[i] = 2 * hd[i] + (hm[2 * k1 + j][i] - 1);
      }
      if (hd[i] != -1)
      {
         for (j = 0; j < 2; ++j)
         {
            if (hm[2 * k2 + j][i] == 0)
            {
               hd[i] = -1;
               break;
            }
            hd[i] = 2 * hd[i] + (hm[2 * k2 + j][i] - 1);
         }
      }
   }
   return;
}

void tagsort (double *data, int *indices, int size)
{
   int   k, indexofmin, pass;

   for (k = 0; k < size; k++)
      indices[k] = k;

   for (pass = 0; pass < size - 1; pass++)
   {
      indexofmin = pass;

      for (k = pass + 1; k < size; k++)
         if (data[indices[k]] < data[indices[indexofmin]])
            indexofmin = k;

      if (indexofmin != pass)
         swap (indices + pass, indices + indexofmin);
   }
   return;
}

void setAllStates (unsigned int *allowed, int n)
{
   /* set all states to allowed */
   int   i;
   for (i = 0; i < n; i++)
   {
      allowed[i] = (1 << 15) - 1;
   }
   return;
}

void setAutozygStates (int *statesAB, unsigned int *allowedAC, int flipAB,
                       int n)
{
   int   i;
   int  *lookupAB;
   lookupAB = flipAB ? flippedStates : unflippedStates;
   for (i = 0; i < n; i++)
   {
      allowedAC[i] &= autozygMasks[lookupAB[statesAB[i]]];
   }
   return;
}

void setTrioStates (int *statesAB, int *statesBC, unsigned int *allowedAC,
                    int flipAB, int flipBC, int n)
{
   int   i;
   int  *lookupAB, *lookupBC;
   lookupAB = flipAB ? flippedStates : unflippedStates;
   lookupBC = flipBC ? flippedStates : unflippedStates;
   for (i = 0; i < n; i++)
   {
      allowedAC[i] &= trioMasks[lookupAB[statesAB[i]]][lookupBC[statesBC[i]]];
   }
   return;
}

void setAutozygosityLabels (int *statesAB, int **labelsA, int flipAB, int n)
{
   int   i;
   int  *lookupAB;
   lookupAB = flipAB ? flippedStates : unflippedStates;
   for (i = 0; i < n; i++)
   {
      if (lookupAB[statesAB[i]] < 5)
         labelsA[1][i] = labelsA[0][i];
   }
}

void setIbdLabels (int *statesAB, int **labelsA, int **labelsB, int n)
{
   int   i;
   for (i = 0; i < n; i++)
   {
      switch (statesAB[i])
      {
          case 0:                      /* 1111 */
          case 8:                      /* 1212 */
             labelsB[0][i] = labelsA[0][i];
             labelsB[1][i] = labelsA[1][i];
             break;
          case 2:                      /* 1112 */
          case 10:                     /* 1213 */
             labelsB[0][i] = labelsA[0][i];
             break;
          case 3:                      /* 1131 */
          case 13:                     /* 1232 */
             labelsB[1][i] = labelsA[1][i];
             break;
          case 5:                      /* 1211 */
             labelsB[0][i] = labelsB[1][i] = labelsA[0][i];
             break;
          case 6:                      /* 1222 */
             labelsB[0][i] = labelsB[1][i] = labelsA[1][i];
             break;
          case 9:                      /* 1221 */
             labelsB[0][i] = labelsA[1][i];
             labelsB[1][i] = labelsA[0][i];
             break;
          case 12:                     /* 1223 */
             labelsB[0][i] = labelsA[1][i];
             break;
             /* nothing for 1122, 1123, 1233, 1234 */
      }
   }
}

void progBar (int it, int n, double fraction)
{
   int   i, prog = ceil (fraction * 50);
   printf ("%cRealization %d of %d [", 13, it, n);
   for (i = 0; i < prog; i++)
   {
      printf ("-");
   }
   for (; i < 50; i++)
   {
      printf (" ");
   }
   printf ("]");
   fflush (stdout);
}
