/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*******************************************************************************

  program name: lm_linkage.c (Estimate or calculate LodScore by component.
                Components can be input in any order)

 (LPT Jul 2007)

  Note that this program is modified and very similar to lm_markers.c.
  The differences are that
  (1) this program allows exact calculation of LodScores for a small pedigree
      with no. of meiosis less than or equal to MaxNMexact, which is a global
      in /Headers/msample.h and set at the start of the program.
  (2) this program performs additional multiple meioses updating: complete and
      restricted individual, sib and 3-generation updating. The constant
      MAXNMupdate, defined in /Headers/mulmsample.h, determines wether to
      perform a complete updating or a restricted one. The constants PMEIO,
      PIND, PSIB, P3GEN are probabilities for each type of updating. They
      are defined in /Headers/mulmsample.h and their sum must be 1.0.
   (These consts are all reaplaced by global ints, set at the start of
     the program: EAT  2007-8)

  The state (X) consists of segregation indicators -- so can generalize to
        cases of more than one observed/pedigree, and multiple loci.

  The data, Y, is the multilocus genotypes of observed individuals
     in the pedigree.

  Obtained by modification of jnt_auto.c.  December 1998.  (EAT)

Here is the plan!
    OK this is the PLAN  (12/15/98)

OK, now we have a general autozyg program, we need to go back to
lod estimation, for those as want.  We use the same program (modified).

Since the data are locus-by-locus, the likelihood ratio contribution is
just ratio of P_theta(S)/P_sim_th(S).

This is very easy, once we get the chromosome set-ups fixed.

1) Trait locus data set-up -- assume trait-locus genotypes for observed
individuals -- done  12/20/98
2) Likelihood-ratio contributions routine,  -- done 12/18/98
3) Simulation and evaluation chromosomes for each location. -- done 12/23/98

Now to integrate the L-sampler, using the new Lsampler and Msampler
      libraries and new routines;        EAT March 2000

Converted to MORGAN_V2.3                 MAJ June 2000

Modifications to specify simulation points, within each interval and
outside of map.    EAT, then converted to MORGAN 2.3 by MAJ, Nov 2000.

Modification of full_seg_prob, and recombinant counts     EAT  Nov 2000
    scoring of counts and tot_log_probs by component and locus/interval.

Modifications to lik_rat, do_lsmpl, etc.                  EAT  Dec 2000

Modifications to use specify intervals for l-ratio updating and use
    more efficient seg_prob updating routines for l-sample steps
                                                          EAT Jan 2001

Added changes for sequential imputation set-up            AWG March 2001

But now we can use locus_mjrpeel with sample_flag zero
     to do a Lange-style lod score using only sampled S_M    EAT May 2001
  We can use the extval and intval positions as computation positions
    -- except we can also compute at the loci, maybe??

This is the version to test new trait stuctures.
Program renamed lm_markers.                                 EAT March 2002

Version modified for quantitative traits (lm_quant.c)       EAT Jan 2004

Output scores file added for CDLL output                    EAT Apr 2004

Exact calculation for small pedigrees added                 LPT Feb 2006

Multiple meioses updating for MCMC added                    LPT Apr 2006

Multiple meiosis updating and exact calculation cleaned up  EAT Sep 2006
  Dependence on new MulMSample versions of setupAutozyg()
   and setupMaps() is removed: these shd now be obsolete.

Program renamed lm_linkage.                                 SML June 2011

*******************************************************************************/

/**
 * @defgroup lm_markers lm_markers
 * @ingroup lodscore
 * @{
 */

#define PNAM  "lm_markers"
#define MARKERS

#include "lodscore.h"

int main (int argc, char *argv[])
{
   FILE *oScorF = NULL;
   FILE *oSeedF;
   CTP_PTR p_ctp;

   double *overall_lod, *overall_sd;
   double *comp_lod, *comp_sd;
   double **prob;

   int   nCompnt1;                      /* Number of components to be         */
   /*                                      calculated exactly.                */
   int   comp_no;                       /* Current component number.          */
   int   ntp;
   int   p1;

   /* ................................  SET-UP   ............................ */

   PCleanUp = clean_auto;
   atexit (clean_morg);

   NSw = setupAutozyg (argc, argv, PNAM, 0);
   setupSampling ();

   nCompnt1 = 0;
   if (!GlobalMcmc)
   {
      /* Count the number of components to be calculated exactly.             */
      for (comp_no = 1; comp_no <= nCompnt; ++comp_no)
         if (cmplist[comp_no].nmeioses <= MaxNMexact)
            ++nCompnt1;
   }

   /* The call to setupInherit was moved down, so we can know whether exact   */
   /* computation is to be done for any of the components.                    */

   if (nCompnt1 == 0)
      prob = setupInherit (1);
   else
      prob = setupInherit (0);

   if (MMSampler == Y)
   {
      /* Set up for the multiple-meiosis sampler.                             */
      Max_update = 8;
      P_meio = 0.2;
      P_ind = 0.3;
      P_sib = 0.3;
      P_3gen = 0.2;

      setup_marriage ();
      prnt_marriage ();
   }

   else if (MMSampler == N)
   {
      /* Set up for the old M-sampler.                                        */
      Max_update = 1;
      P_meio = 1.0;
      P_ind = 0.0;
      P_sib = 0.0;
      P_3gen = 0.0;
   }

   /* Now set evaluation points for lange_lik.                                */
   /* ntp is number of trait positions plus number of markers (NLocPos + 1).  */

   p_ctp = alloc_ctp (NLocPos + 1);

   ntp = set_eval_points (NMarks, p_ctp);

   if (ntp < NLocPos + 1)
      realloc_ctp (ntp, p_ctp);

   /* Now estimate lod scores using MCMC for large pedigrees and find exact   */
   /* lod scores for small pedigrees.                                         */

   overall_lod = alloc_dbl1d (ntp);
   overall_sd = alloc_dbl1d (ntp);

   if (!GlobalMcmc)
   {
      comp_lod = alloc_dbl1d (ntp);
      comp_sd = alloc_dbl1d (ntp);

      for (comp_no = 1; comp_no <= n_compnts; ++comp_no)
      {
         printf ("\n\n");               /* Two blank lines before component.  */

         if (cmplist[comp_no].nmeioses == 0)
         {
            print_zero_meioses_message (comp_no);
            continue;
         }

         printf (" ====== LodScore for component %d is ", comp_no);

         if (cmplist[comp_no].nmeioses <= MaxNMexact)
         {
            printf ("calculated exactly ======\n\n");
            fflush (stdout);

            exact_lods (comp_no, NMarks, nFGL, ntp, comp_lod, p_ctp);

            printf ("\n Summary for component %d:", cmplist[comp_no].index);
            printf ("\n Base log-likelihood log_10 P(Y_T) = %e\n", comp_lod[0]);
            prntLodSHead ("LodScore by exact computation");
         }
         else
         {
            printf ("estimated using MCMC ======\n\n");

            mcmc_lods (comp_no, MMSampler, ntp, p_ctp, prob, comp_lod, comp_sd,
                       &oScorF);

            prntLodSHead
               ("LodScore estimates by Rao-Blackwellized computation");
         }
         prntLodScores (ntp, comp_lod, comp_sd);

         /* Accumulate the overall lod scores with their standard deviations. */
         for (p1 = 1; p1 < ntp; ++p1)
         {
            overall_lod[p1] += comp_lod[p1];
            overall_sd[p1] += comp_sd[p1];
         }

         /* Re-initialize comp_sd to zeroes before computing the lod scores   */
         /* for the next component.                                           */
         memset (comp_sd, 0.0, ntp * sizeof (double));
      }

      free_1d ((void *) &comp_lod);
      free_1d ((void *) &comp_sd);

      /* Now print the overall cumulative lod scores unless there was just a  */
      /* single component in the pedigree.                                    */

      if (nCompnt > 1)
      {
         printf ("\n\n === LodScore for all the components completed ===");
         printf ("\n\n");
         fflush (stdout);

         prntLodSHead ("LodScore estimates through all components");
         prntLodScores (ntp, overall_lod, overall_sd);
      }
   }

   else if (GlobalMcmc)
   {
      /* For global MCMC, use mcmc_lods with the first parameter set to 0.    */
      mcmc_lods (0, MMSampler, ntp, p_ctp, prob, overall_lod, overall_sd,
                 &oScorF);

      printf ("\n\n === LodScore for all the components completed ===");
      printf ("\n\n");
      fflush (stdout);

      prntLodSHead ("LodScore estimates by Rao-Blackwellized computation");
      prntLodScores (ntp, overall_lod, overall_sd);
   }

   /* If an output scores file was being produced, it can now be closed.      */

   if (FilIt && oScorF)
      fclose (oScorF);

   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   /* Free up allocated memory.                                               */

   if (TraitTyp == QUANTITATIVE)
      free_input_data ();

   free_seg_pen (NLoci, &prob);
   free_chromo (0);

   if (MMSampler == Y)
   {
      free_msmpl2 (NLoci);
      free_marriage ();
   }

   free_1d ((void *) &overall_lod);
   free_1d ((void *) &overall_sd);
   free_ctp (p_ctp);

   /* Now we are done.                                                        */

   exit (EXIT_SUCCESS);
}
