/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*                                                                            *
 *       pr_log_prob_seg(), pr_log_prob_segl (), pr_log_prob_pen(),           *
 *       pr_reco_counts()                                                     *
 *                                                                            *
 */

#include "markers.h"
#include "traits.h"

void pr_log_prob_seg (int loc1, int nloc, double ns, double *tot_log_pr_seg,
                      double tcount)
{
   int   i, j;

   if (PrintLevel >= 4)
   {
      printf ("    Average total log-probability of segregations:\n");
      printf ("       for first locus (constant): %8.3f\n"
              "       for each successive locus:", ns * log (2.0));

      for (i = 0, j = loc1; j < nloc; ++i, ++j)
      {
         if (!(i % 8))
            printf ("\n       ");
         printf (" %8.3f", tot_log_pr_seg[j] / tcount);
      }
      printf ("\n\n");
      fflush (stdout);
   }

   return;
}

void pr_log_prob_segl (int loc1, int nloc, double ns, double *tot_log_lf_seg,
                       double *tot_log_pr_seg, double *tot_log_rt_seg,
                       double tcount)
{
   int   j;

   if (PrintLevel >= 4)
   {
      printf ("    Average total log-probability of segregations:\n");
      printf ("       for first locus (constant): %8.3f\n"
              "       for each successive locus:\n", ns * log (2.0));
      printf ("                   left        sim      right\n");

      for (j = loc1; j < nloc; ++j)
      {
         printf ("     %3d,%3d ", chromo[0][j], chromo[0][j + 1]);
         printf (" %9.3f  %9.3f  %9.3f\n", tot_log_lf_seg[j] / tcount,
                 tot_log_pr_seg[j] / tcount, tot_log_rt_seg[j] / tcount);
      }
      printf ("\n");
      fflush (stdout);
   }

   return;
}

void pr_log_prob_pen (int loc1, int nloc, double *tot_log_pr_pen, int tcount)
{
   int   i, j, ncol, wcol;

   if (PrintLevel >= 4)
   {
      get_colfmt ((LineWidth - 7), LocNamLen, 8, 1, &ncol, &wcol);

      printf ("    Average total log-probability of penetrances, by locus:");

      for (i = 0, j = loc1; j <= nloc; ++i, ++j)
      {
         if (!(i % ncol))
            printf ("\n      ");
         printf (" %*s", wcol, traits[chromo[0][j]].self);
      }
      for (i = 0, j = loc1; j <= nloc; ++i, ++j)
      {
         if (!(i % ncol))
            printf ("\n      ");
         printf (" %*.3f", wcol, tot_log_pr_pen[chromo[0][j]] / tcount);
      }
      printf ("\n\n");
      fflush (stdout);
   }

   return;
}

void pr_reco_counts (int loc1, int nloc, int **recnts)
{
   int   i, j;

   if (PrintLevel >= 4)
   {
      printf ("    Recombination counts for map intervals:\n");
      printf ("                  males:                       females:\n");
      printf ("                  total    recomb   non-rec   "
              "   total    recomb   non-rec\n");

      for (j = loc1; j < nloc; ++j)
      {
         printf ("     %3d,%3d ", chromo[0][j], chromo[0][j + 1]);
         for (i = 0; i < 6; ++i)
         {
            if (i == 3)
               printf (" ");
            printf ("%10d", recnts[j][i]);
         }
         printf ("\n");
      }
      printf ("\n");
      fflush (stdout);
   }

   return;
}
