/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "markers.h"

static double *basereal = NULL;
static int *baseint = NULL;

static double *qset = NULL;

/* sum_gene and stuff is deleted,  01/2005  EAT  */

/* new routines for fuzzy-pvals :
 * alloc_fzp(), score_all(), scor_pval() and norcum()
 * added here  01/2005  EAT
 */

/*  measure-specific stuff removed from scoring (histograms etc.)
 *  setup for ibd_measure scoring routines is implemented/improved
 *  EAT   07/2005
 */

/* added temp (??) code in score_all to compute values of expectation
   statistics  EAT  10/2005  */
void score_all (Measure scoraff, double **fzpval, int nloc, int nnull,
                double **qqnull, double *totstat)
{
   int   c, jj;
   double lx, mx;

   lx = -1000.0;

   for (jj = 1; jj <= nloc; ++jj)
   {
      mx = 0.0;
      for (c = 0; c < n_compnts; ++c)
         if (cmplist[c + 1].naff)
         {
            mx += scoraff (c, jj, baseint, basereal);
         }
      totstat[jj] += mx;
      scor_pval (mx, nnull, fzpval[jj], qqnull[jj]);
      if (mx > lx)
         lx = mx;
   }
   totstat[0] += lx;
   scor_pval (lx, nnull, fzpval[0], qqnull[0]);

   return;
}

void scor_pval (double t, int nn, double *fzp, double *qnull)
{
   int   bgg, eqq, i;
   bgg = 0;
   eqq = 0;

   for (i = 0; i < nn; ++i)
   {
      if (qnull[i] > t)
         ++bgg;
      if (qnull[i] == t)
         ++eqq;
   }

   for (i = bgg; i <= bgg + eqq; ++i)
      fzp[i] += 1.0 / (eqq + 1.0);

   return;
}

void norcum (int nt, int nn, double *fzp)
{
   int   j;
   fzp[0] = fzp[0] / nt;
   for (j = 1; j < nn; ++j)
      fzp[j] = fzp[j - 1] + fzp[j] / nt;

   return;
}

int ***alloc_affhist (void)
/* allocs and initializes (to 0) hist[nc][nl][naff],
 * where naff is the number of affected individuals for the component.
 * returns hist[][][]                                                         */
{
   int ***h, c;

   h = malloc (nCompnt * sizeof (int **));
   alloc_chk (WHERE, h);

   for (c = 0; c < nCompnt; ++c)
      if (cmplist[c + 1].naff)
         h[c] = alloc_int2d (NLoci, cmplist[c + 1].naff);
      else
         h[c] = NULL;

   return h;
}

void setup_ibdmeas (Measure scoraff)
/* sets up up any default necessary values for int and real arrays
   in the measures: note spairs and robdom corrected for coherence.           */
{

   if (!baseint)
   {
      baseint = alloc_int1d (1);
      baseint[0] = 0;
   }

   if (!basereal)
   {
      basereal = alloc_dbl1d (2);
      basereal[0] = 0.0;
      basereal[1] = 7.0;
   }
}

void score_aff (Measure scoraff, int ***hist)
{
   int   c, jj, mx;

   for (c = 0; c < nCompnt; ++c)
   {
      if (cmplist[c + 1].naff)
         for (jj = 0; jj < NLoci; ++jj)
         {
            mx = scoraff (c, jj, baseint, basereal);
            ++hist[c][jj][mx - 1];      /* 0 < mx <= cmplist[c+1].naff  */

         }
   }

   return;
}

void multi_score (Measure scoraff, int ***hist)
/* modified from score_aff to store max score over loci in 0 component.
   01/2005  EAT */
{
   int   c, jj, mx, lx;

   for (c = 0; c < nCompnt; ++c)
   {
      lx = 1;
      if (cmplist[c + 1].naff)
         for (jj = 1; jj < NLoci; ++jj)
         {
            mx = scoraff (c, jj, baseint, basereal);
            ++hist[c][jj][mx - 1];      /* 0 < mx <= cmplist[c+1].naff  */
            if (mx > lx)
               lx = mx;
         }
      ++hist[c][0][lx - 1];
   }

   return;
}

void multi_gen_score (int t, Measure scoraff, int numpos, double **hdsn)
/* modified from score_aff to store max score over loci in 0 component.
   and to use a general score function, to be evaluated at arb. points     */
/* Edited from multi_score to get rid of histograms   etc.   EAT  07/2005  */
/* Evaluation at numpos positions  (often nloc) + 1 for max value          */
/* Values stored in array  hdsn[1:numpos] -- called for each realization   */
/*    Max value stored in hdsn[0]                                          */
{
   int   c, jj;
   double mx, lx;

   lx = -1000.0;
   for (jj = 1; jj <= numpos; ++jj)
   {
      mx = 0.0;
      for (c = 0; c < nCompnt; ++c)
      {
         if (cmplist[c + 1].naff)
            mx += scoraff (c, jj, baseint, basereal);
      }
      hdsn[jj][t] = mx;
      if (mx > lx)
         lx = mx;
   }
   hdsn[0][t] = lx;

   return;
}

/*  Old histogram version: replaced  July 2005, EAT
 * void make_quantiles (int iq, int dd, int mh, int *h, double *dsn)
 *  iq = number of points in the quantile dsn (typically 1000)
 *  dd = number of realizations in histogram per quantile point
 *  h is the histogram to be converted, and dsn the quantiles to be set
 *  mh is the mx value in the histogram.

 *  Note in cell h[k], score is k+1  !!                        */
/* {
 *    int   i, j, csum;
 *    csum = 0;
 *    j = 0;
 *    printf (" now in make_quantiles: iq=%d, decr=%d, mh = %d\n", iq, dd, mh);
 *    fflush (stdout);
 *    for (i = 0; i < mh; ++i)
 *    {
 *       csum += h[i];
 *       while ((j + 1) * dd <= csum)
 *       {
 *          dsn[j] = i + 1;
 *          ++j;
 *       }
 *    }
 *    return;
 * }
 */

void make_quantiles (int iq, int dd, double *h, double *dsn)
/* iq = number of points in the quantile dsn (typically 1000)  */
/* dd = number of realizations in total list of values per quantile point */
/*  h is the total list of values (unsorted) to be put into dsn quantiles */
{
   int   i;
   i = dd * iq;
   qsort (h, i, sizeof (double), qfunc_doub);
   for (i = 0; i < iq; ++i)
      dsn[i] = h[i * dd + dd / 2];
   dsn[iq] = h[iq * dd - 1];

   return;
}

void compress_marginals (int m1, int m2, int nn, double **dsn)
{
   int   i, j, nd, med;

   if (m1 >= m2)
      return;
   nd = (m2 + 1 - m1);
   med = 0;
   if ((nd / 2) * 2 != nd)
      med = nd / 2;

   if (qset == NULL)
      qset = alloc_dbl1d (m2 + 1);

   for (i = 0; i <= nn; ++i)
   {
      for (j = 0; j < nd; ++j)
         qset[j] = dsn[m1 + j][i];
      qsort (qset, nd, sizeof (double), qfunc_doub);
      if (med)
         for (j = 0; j < nd; ++j)
            dsn[m1 + j][i] = qset[med];
      else
         for (j = 0; j < nd; ++j)
            dsn[m1 + j][i] = 0.5 * (qset[nd / 2] + qset[(nd / 2) - 1]);
   }

   return;
}

void pr_quantiles (int iq, int dd, int prq, int nd, double **dsn)
/* iq = # quantiles, dd= number of MC realizations, prq=print interval */
/* nd = number of dsns in dsn array                                    */
{
   int   i, j, n;

   printf (" Quantiles at 1/%d from %d realizations", iq, dd);
   printf (" to be printed at intervals of %d:\n\n", prq);

   n = iq / prq;
   for (i = 0; i <= n; ++i)
   {
      printf (" %d ", prq * i);
      for (j = 0; j < nd; ++j)
         printf (" %4.1f", dsn[j][prq * i]);
      printf ("\n");
   }
   printf ("\n");

   return;
}

void free_score_aff (void)
{
   free_1d ((void *) &basereal);
   free_1d ((void *) &baseint);
   free_1d ((void *) &qset);

   return;
}
