/******************************************************************************/
/*                                                                            */
/* These subroutines are called from proc_ibd().                              */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* The mak_MarLod subroutine sets up a vector of indices to the markers at    */
/* which lodscores are to be computed.  The argument to this subroutine is a  */
/* vmap_node pointer to a selected vmap_node.                                 */
/*                                                                            */
/******************************************************************************/

void mak_MarLod (VMAP_PTR p_vmap)
{
   int   ntpos;                         /* Number of test position indices.   */
   int   m1;                            /* Current marker subscript.          */

   /* Allocate storage for MarLod, which will be used to contain the indices  */
   /* of the markers at which lodscores are to be computed.                   */

   ntpos = max (NMarks, p_vmap->no_marker_nums);
   MarLod = malloc (ntpos * sizeof (int));
   alloc_chk (WHERE, MarLod);

   /* If there are selected marker numbers in MarSel, copy these to MarLod.   */
   /* Otherwise, copy the map test tloc at marker numbers from the vmap_node  */
   /* passed as the argument to this subroutine.                              */

   if (MarSel != NULL)
   {
      for (m1 = 0; m1 < NMarks; ++m1)
         MarLod[m1] = MarSel[m1];
   }

   else
   {
      for (m1 = 0; m1 < p_vmap->no_marker_nums; ++m1)
         MarLod[m1] = p_vmap->p_marker_nums[m1];
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* Checks scalar parameters and sets defaults.                                */
/*                                                                            */
/******************************************************************************/

int test_ibd_pars (void)
{
   if (NIBDgraphs == UNDEF)
   {
      err_typ = ERA_NIG;
      return FALSE;
   }

   if (ScorIt == UNDEF)
      ScorIt = 1;

   if (FilIt == UNDEF)
      FilIt = NIBDgraphs / ScorIt;

   if (err_typ)
      return FALSE;

   return TRUE;
}

void prnt_ibd_pars (void)
{
   if (NIBDgraphs != UNDEF)
   {
      fprintf (stdout, " Number of IBD graphs requested is %d\n", NIBDgraphs);
      fprintf (stdout, "\n");
   }

   if (GL_LODS)
      fprintf (stdout,
               " Number of batches for MC variance estimation is %d\n\n",
               NBtch);

   return;
}
