/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/* Parameter statement functions for "simulate ..." and "select ..."
 * statements, both for markers and for traits, are included here.
 * Also included are the "set trait .. effects ..", "set trait .. tloc .."
 * and "set inheritance .. tloc .." statements.                               */

int sim_mk (void)                       /* simulate markers (all)             */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, NTLOC, SS_MKALL))
         make_sim_node (NCH, NTLOC, SS_MKALL);
   }

   return err_typ;
}

int sim_mk_c (void)                     /* simulate markers (all) on chrom    */
{
   if (chk_stat ())
   {
      if (chk_sim (CH, NTLOC, SS_MKALL))
         make_sim_node (CH, NTLOC, SS_MKALL);
   }

   return err_typ;
}

int sim_tr (void)                       /* simulate traits                    */
{
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_TRLIST))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* tests for existing node     */
            p_sim_tr = alloc_sim_tr_node ();
         else if (p_sim_tr->no_traits)  /* tests for existing trait           */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_TRLIST); /* stores info in node       */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores array of traits.            */
      }
   }

   return err_typ;
}

int sim_tl (void)                       /* simulate tloc                      */
{
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_TLLIST))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* tests for existing node     */
            p_sim_tr = alloc_sim_tr_node ();
         else if (p_sim_tr->no_traits)  /* tests for existing tloc            */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_TLLIST); /* stores info in node       */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores array of traits.            */
      }
   }

   return err_typ;
}

int sim_ibd_reals (void)                /* simulate ... ibd realizations      */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (j_par[0] < 1)
         err_typ = ERR_NIR;
      NRealizations = j_par[0];
   }

   return err_typ;
}

int sel_inh (void)                      /* select inheritance  REV*****       */
{
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_INHER))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* tests for existing node     */
            p_sim_tr = alloc_sim_tr_node ();
         else if (p_sim_tr->no_traits)  /* tests for existing tloc            */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_INHER); /* stores info in node        */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores array of traits.            */
      }
   }

   return err_typ;
}

int sel_mk_all (void)                   /* select all markers                 */
{                                       /* SAME CODE as sim_mk:  REPLACE???   */
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, NTLOC, SS_MKALL))
         make_sim_node (NCH, NTLOC, SS_MKALL);
   }

   return err_typ;
}

int sel_mk_c_all (void)                 /* select all markers on chromosome   */
{                                       /* same code as sim_mk_c:  REPLACE??  */
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (CH, NTLOC, SS_MKALL))
         make_sim_node (CH, NTLOC, SS_MKALL);
   }

   return err_typ;
}

int sel_mk (void)                       /* select markers (list)              */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (NCH, NTLOC, SS_MKLIST))
         make_sim_node (NCH, NTLOC, SS_MKLIST);
   }

   return err_typ;
}

int sel_mk_c (void)                     /* select markers (list) on chrom     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim (CH, NTLOC, SS_MKLIST))
         make_sim_node (CH, NTLOC, SS_MKLIST);
   }

   return err_typ;
}

int sel_tr (void)                       /* select traits                      */
{                                       /* SAME CODE as sim_tr:  RENAME??     */
   SIMTRAIT_PTR p_sim_tr;

   if (chk_stat ())
   {
      chk_repeat ();
      if (chk_sim_tr (SS_TRLIST))
      {
         if (!(p_sim_tr = chk_sim_tr_node ())) /* Tests for an existing node. */
            p_sim_tr = alloc_sim_tr_node ();
         if (p_sim_tr->no_traits)       /* Tests for already existing traits. */
            free_trait_array (p_sim_tr);
         stor_sim_tr_data (p_sim_tr, SS_TRLIST); /* stores select info in nod */
         alloc_trait_array (p_sim_tr);
         stor_trait_array (p_sim_tr);   /* Stores traits into p_trait array.  */
      }
   }

   return err_typ;
}

int sel_stat_tran_matr (void)           /* Select state transition matrix     */
{
   if (chk_stat ())
   {
      chk_repeat ();
      switch (j_par[0])
      {
          case 2009:
             StateTransMat = STM_2009;
             break;
          case 2011:
             StateTransMat = STM_2011;
             break;
          default:
             err_typ = ERR_STM;
             break;
      }
   }

   return err_typ;
}

int sel_phas_data (void)                /* Select phased data                 */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PhaseofData != UNDEF)
         err_typ = ERR_SPD;
      PhaseofData = PHASED;
   }

   return err_typ;
}

int sel_unph_data (void)                /* Select unphased data               */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PhaseofData != UNDEF)
         err_typ = ERR_SPD;
      PhaseofData = UNPHASED;
   }

   return err_typ;
}

int sel_part_phas_data (void)           /* Select partially phased data       */
{
   if (chk_stat ())
   {
      chk_repeat ();
      if (PhaseofData != UNDEF)
         err_typ = ERR_SPD;
      PhaseofData = PARTIALPHASE;
   }

   return err_typ;
}

int set_inh_tl (void)                   /* set inheritances .. tlocs ..       */
{
   INHERTLOC_PTR p_inhtl;
   int   i;

   if (chk_stat ())
      if (chk_set_inh_tl ())
         for (i = 0; i < nk_par / 2; ++i)
         {
            if ((p_inhtl = chk_inh_tl_node (2 * i))) /* Checks existing node  */
               free_inh_tl_array (p_inhtl); /* for inheritance.               */
            else
               p_inhtl = alloc_inh_tl_node (2 * i);
            alloc_inh_tl_array (p_inhtl); /* Adds tloc numbers array.         */
            stor_inh_tl_array (p_inhtl, 2 * i);
         }

   return err_typ;
}

int set_tr_eff (void)                   /* set trait .. effects ..            */
{
   TRAITEFF_PTR p_t;

   if (chk_stat ())
      if (chk_set_tr_eff ())
      {
         if ((p_t = chk_tr_eff_node (0))) /* Checks for exist node for trait. */
            free_tr_eff_array (p_t);
         else
            p_t = alloc_tr_eff_node (0);
         p_t->set_trait_eff_inp = Y;
         alloc_tr_eff_array (p_t);
         stor_tr_eff_array (p_t);       /* Store effects into eff_nums array. */
      }

   return err_typ;
}

int set_tr_tl (void)                    /* set traits ... tlocs ...           */
{
   TRAITTLOC_PTR p_tt;
   int   x1;

   if (chk_stat ())
      if (chk_set_tr_tl (TRLIST_TLLIST))
         for (x1 = 0; x1 < nk_par / 2; ++x1)
         {
            /* Checks for an existing trait_tloc_node.                        */
            if ((p_tt = chk_tr_tl_node (2 * x1)))
               free_tr_tl_array (p_tt);
            else
               p_tt = alloc_tr_tl_node (2 * x1);
            alloc_tr_tl_array (p_tt, TRLIST_TLLIST); /* Adds tlocs array.     */
            stor_tr_tl_array (p_tt, 2 * x1, TRLIST_TLLIST);
         }

   return err_typ;
}

int set_tr_tl_mlt (void)                /* set trait ... multiple tlocs ...   */
{
   TRAITTLOC_PTR p_tt;

   if (chk_stat ())
      if (chk_set_tr_tl (TR_TLLIST))
      {
         /* Checks for an existing trait_tloc_node.                           */
         if ((p_tt = chk_tr_tl_node (0)))
            free_tr_tl_array (p_tt);
         else
            p_tt = alloc_tr_tl_node (0);
         alloc_tr_tl_array (p_tt, TR_TLLIST);
         stor_tr_tl_array (p_tt, 0, TR_TLLIST); /* Stores tloc numbers.       */
      }

   return err_typ;
}
