/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*
 * proc_seeds() calls functions to set parameters for seeds
 */

int proc_seeds (void)
{
   if (!test_seeds ())
      if (proc_err ())
         return FALSE;

   if (!test_oseeds ())
      if (proc_err ())
         return FALSE;

   if (n_err)
      return proc_end ();

   return TRUE;
}

/******************************************************************************/
/*                                                                            */
/* The default_marker_seeds subroutine checks to see if the marker seeds have */
/* been set.  If not, then this subroutine sets the marker seeds to the       */
/* defaults and sets err_typ so that the user will be informed that the       */
/* defaults were used.                                                        */
/*                                                                            */
/******************************************************************************/

void default_marker_seeds (void)
{
   if (!MarkSeedSet)                    /* Tests for marker seeds set.        */
   {
      MarkSeedC = CONGRUENTIAL_SEED;
      MarkSeedT = TAUSWORTHE_SEED;
      err_typ = ERD_SSD;
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* The default_sampler_seeds subroutine checks to see if the sampler seeds    */
/* have been set.  If not, then this subroutine sets the sampler seeds to the */
/* defaults and sets err_typ so that the user will be informed that the       */
/* defaults were used.                                                        */
/*                                                                            */
/******************************************************************************/

void default_sampler_seeds (void)
{
   if (!SamplSeedSet)                   /* Tests for sampler seeds set.       */
   {
      SamplSeedC = CONGRUENTIAL_SEED;
      SamplSeedT = TAUSWORTHE_SEED;
      err_typ = ERA_SSS;
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* The default_trait_seeds subroutine checks to see if the trait seeds have   */
/* been set.  If not, then this subroutine sets the trait seeds to the        */
/* defaults and sets err_typ so that the user will be informed that the       */
/* defaults were used.                                                        */
/*                                                                            */
/******************************************************************************/

void default_trait_seeds (void)
{
   if (!TraitSeedSet)                   /* Tests for trait seeds set.         */
   {
      TraitSeedC = CONGRUENTIAL_SEED;
      TraitSeedT = TAUSWORTHE_SEED;
      err_typ = ERD_STD;
   }

   return;
}

/******************************************************************************/
/*                                                                            */
/* The default_permutation_seeds subroutine checks to see if the permutation  */
/* seeds have been set.  If not, then this subroutine sets the permutation    */
/* seeds to the defaults and sets err_typ so that the user will be informed   */
/* that the defaults were used.                                               */
/*                                                                            */
/******************************************************************************/

void default_permutation_seeds (void)
{
   if (!PermSeedSet)                    /* Tests for permutation seeds set.   */
   {
      PermSeedC = CONGRUENTIAL_SEED;
      PermSeedT = TAUSWORTHE_SEED;
      err_typ = ERA_SSP;
   }

   return;
}

/* --------- test fncs:  test results of processing seed statements --------- */

int test_seeds (void)
/* Determines whether seeds were given.                                       */
{
   if (GENEDROP)
   {
      default_marker_seeds ();

      if (!TraitSeedSet)                /* Tests for trait seeds set.         */
      {
         TraitSeedC = 0;
         TraitSeedT = 0;
      }
   }

   if (MARKERDROP)
   {
      default_trait_seeds ();

      if (!MarkSeedSet)                 /* Tests for marker seeds set.        */
      {
         MarkSeedC = 0;
         MarkSeedT = 0;
      }
   }

   if (LM_IBDTESTS)
   {
      if (PermutEnable)                 /* Tests for permutation tests.       */
      {
         default_permutation_seeds ();

         if (!SamplSeedSet)             /* Tests for sample seeds set.        */
         {
            SamplSeedC = 0;
            SamplSeedT = 0;
         }
      }
      else                              /* Sets sampling seeds only.          */
      {
         PermSeedC = 0;
         PermSeedT = 0;

         default_sampler_seeds ();
      }
   }

   if (LM_TWOQTL)
   {
      default_trait_seeds ();
      default_sampler_seeds ();
   }

   if (IBD_HAPLO)
   {
      if (NRealizations > 0)            /* Check that sampling is being done. */
         default_sampler_seeds ();
   }

   if (IBD_CREATE)
   {
      if (SubProgChoice == 'f' || SubProgChoice == 'p' || SubProgChoice == 'q')
         default_sampler_seeds ();
   }

   if (IBDDROP || LM_SCHNELL || LM_MAP || LM_PVAL || LM_AUTO
       || LM_LODS || LM_BAYES || LM_MARKERS)
   {
      default_sampler_seeds ();
   }

   if (!err_typ)
      return TRUE;
   else
      return FALSE;
}

int test_oseeds (void)
{
   pft = get_filent (OSEED);

   if (GENEDROP || MARKERDROP)
   {
      if (pft->fname && !MarkSeedsOut && !TraitSeedsOut)
      {
         if (NLoci - NTraits)
            MarkSeedsOut = Y;
         if (NTraits)
            TraitSeedsOut = Y;
      }
      if (!pft->fname && MarkSeedsOut)
      {
         MarkSeedsOut = N;
         err_typ = ERD_OSM;
      }
      if (!pft->fname && TraitSeedsOut)
      {
         TraitSeedsOut = N;
         err_typ = ERD_OST;
      }
   }

   else if (IBDDROP || LM_SCHNELL || LM_MAP || LM_PVAL || LM_AUTO || LM_LODS
            || LM_BAYES || LM_MARKERS || IBD_HAPLO || IBD_CREATE || LM_TWOQTL)
   {
      if (pft->fname)
         SamplSeedsOut = Y;
      else
         SamplSeedsOut = N;
   }

   else if (LM_IBDTESTS)
   {
      if (pft->fname && !SamplSeedsOut && !PermSeedsOut)
      {
         SamplSeedsOut = Y;
         PermSeedsOut = Y;
      }
      if (!pft->fname && SamplSeedsOut)
      {
         SamplSeedsOut = N;
         err_typ = ERA_USS;
      }
      if (!pft->fname && PermSeedsOut)
      {
         PermSeedsOut = N;
         err_typ = ERA_USP;
      }
   }

   if (!err_typ)
      return TRUE;
   else
      return FALSE;
}
