/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "lsample.h"
#include "sample_order.h"

static unsigned long *req_set[3];

static void check_family (struct Family *ff, int *gt[2], int *fg[2],
                          const int type)
{
   int   i, j, k, k1, pg[2], mg[2], ids, idd, tt[2], tt1[2], flag;
   Ind  *kid;
   Indlist *id_list;
   unsigned long a, a1, b[2], b1[2], c;

   /* Get parental genotypes */
   ids = ff->sire->index;
   idd = ff->dam->index;
   for (i = 0; i < 2; i++)
   {
      pg[i] = gt[i][ids] - 1;
      mg[i] = gt[i][idd] - 1;
   }
   if (ff->sire->pa->index == INVALID_INDEX)
   {
      if (pg[X_MAT] == pg[X_PAT])
         fg[0][ids] |= ORDER_OK;
      fg[X_MAT][ids] |= SEG_UNDEF;
      fg[X_PAT][ids] |= SEG_UNDEF;
   }
   if (ff->dam->pa->index == INVALID_INDEX)
   {
      if (mg[X_MAT] == mg[X_PAT])
         fg[0][idd] |= ORDER_OK;
      fg[X_MAT][idd] |= SEG_UNDEF;
      fg[X_PAT][idd] |= SEG_UNDEF;
   }
   b[X_MAT] = (1 << mg[0]) | (1 << mg[1]);
   b[X_PAT] = (1 << pg[0]) | (1 << pg[1]);
   flag = 0;
   if (((fg[0][idd] | fg[1][ids]) & (ORDER_OK | ORDER_REV)) == ORDER_OK)
   {
      b1[X_MAT] = (1 << mg[X_MAT]) | (1 << pg[X_MAT]);
      b1[X_PAT] = (1 << mg[X_PAT]) | (1 << pg[X_PAT]);
      flag = 1;
   }
   /* Check if offspring order is defined */
   id_list = ff->off;
   while (id_list)
   {
      kid = id_list->current;
      j = kid->index;
      k = gt[0][j] - 1;
      k1 = gt[1][j] - 1;
      a = 1 << k;
      a1 = 1 << k1;
      if (type == MARKER_LOCUS)
      {
         c = req_set[0][j];
         if (a & c)
            a = c;
         c = req_set[1][j];
         if (a1 & c)
            a1 = c;
      }
      for (i = 0; i < 2; i++)
      {
         tt[i] = ((a & b[i]) && (a1 & b[i ^ 1])) ? 1 : 0;
      }
      if (!(tt[X_MAT] || tt[X_PAT]))
      {
         fprintf (stderr, "Internal error - bad configuration\n");
         exit (EXIT_FAILURE);
      }
      if (tt[0] && tt[1])
         if (gt[X_MAT][j] == gt[X_PAT][j])
            fg[0][j] = ORDER_OK;
      if (flag)
      {
         tt1[0] = tt1[1] = 0;
         if (tt[0])
            for (i = 0; i < 2; i++)
               tt1[i] = (a & (1 << mg[i])) ? 1 : 0;
         if (tt[1])
            for (i = 0; i < 2; i++)
               tt1[i] |= (a & (1 << pg[i])) ? 1 : 0;
         if (tt1[0])
            fg[0][j] |= SEG_MAT_OK;
         if (tt1[1])
            fg[0][j] |= SEG_PAT_OK;
         tt1[0] = tt1[1] = 0;
         if (tt[0])
            for (i = 0; i < 2; i++)
               tt1[i] = (a1 & (1 << mg[i])) ? 1 : 0;
         if (tt[1])
            for (i = 0; i < 2; i++)
               tt1[i] |= (a1 & (1 << pg[i])) ? 1 : 0;
         if (tt1[0])
            fg[1][j] |= SEG_MAT_OK;
         if (tt1[1])
            fg[1][j] |= SEG_PAT_OK;
      }
      if (tt[0])
         fg[0][j] |= ORDER_OK;
      if (tt[1])
         fg[0][j] |= ORDER_REV;
      if (tt[1] && !tt[0])
      {
         fprintf (stderr,
                  "Sampled genotyped reversed - this should not happen!\n");
         exit (EXIT_FAILURE);
      }
      id_list = id_list->next;
   }
}

/* Check which orders and segs are fixed by the unordered
 * genotypes of locus */
void check_fixed (struct Locus *locus)
{
   int   id, i, i1, j, k, fam, comp, *seg[2], *gt[2], s;
   struct Family *ff;

   printf ("In check_fixed(): %d\n", locus->index);
   for (i = 0; i < 2; i++)
   {
      seg[i] = locus->seg[i];
      gt[i] = locus->gt[i];
   }
   for (i = 0; i < 3; i++)
      req_set[i] = locus->req_set[i];
   for (k = id = comp = 0; comp < n_compnts; comp++)
   {
      printf ("comp %d, size = %d, nfam = %d\n", comp, cp_size[comp],
              nfam[comp]);
      for (i = 0; i < cp_size[comp]; i++)
      {
         j = cplist[id + i];
         seg[0][j] = seg[1][j] = 0;
      }
      for (fam = 0; fam < nfam[comp]; fam++)
      {
         ff = famlist + (k++);
         check_family (ff, gt, seg, locus->locus_type);
      }
      for (i = 0; i < cp_size[comp]; i++)
      {
         j = cplist[id + i];
         printf ("%d", nghd[j].self);
         for (i1 = 0; i1 < 2; i1++)
         {
            s = seg[i1][j];
            printf ("  %d", gt[i1][j]);
            if (s & SEG_MAT_OK)
               fputs (" SEG_MAT_OK", stdout);
            if (s & SEG_PAT_OK)
               fputs (" SEG_PAT_OK", stdout);
            if (s & ORDER_OK)
               fputs (" ORDER_OK", stdout);
            if (s & ORDER_REV)
               fputs (" ORDER_REV", stdout);
            if (s & SEG_UNDEF)
               fputs (" SEG_UNDEF", stdout);
         }
         printf ("\n");
      }
      id += cp_size[comp];
   }
   exit (0);
}
